@@ -2,14 +2,16 @@ package OpenGuides::Utils;
 
 use strict;
 use vars qw( $VERSION );
-$VERSION = '0.10';
+$VERSION = '0.21';
 
 use Carp qw( croak );
 use Wiki::Toolkit;
 use Wiki::Toolkit::Formatter::UseMod;
-use Wiki::Toolkit::Plugin::RSS::Reader;
 use URI::Escape;
 use MIME::Lite;
+use Net::Netmask;
+use List::Util qw( first );
+use Data::Validate::URI qw( is_web_uri );
 
 =head1 NAME
 
@@ -57,7 +59,8 @@ dbname
 
 =item *
 
-indexing_directory - for the L<Search::InvertedIndex> or L<Plucene> files to go
+indexing_directory - for the L<Search::InvertedIndex>, L<Plucene>,
+or C<Lucy> files to go in
 
 =back
 
@@ -94,14 +97,25 @@ sub make_wiki_object {
 
     # Make search.
     my $search;
-    if ( $config->use_plucene
+    if ( $config->use_lucy ) {
+        $search = $class->make_lucy_searcher( config => $config );
+    } elsif ( $config->use_plucene
          && ( lc($config->use_plucene) eq "y"
               || $config->use_plucene == 1 )
        ) {
         require Wiki::Toolkit::Search::Plucene;
-        $search = Wiki::Toolkit::Search::Plucene->new(
-                                       path => $config->indexing_directory,
-                                                 );
+        my %plucene_args = ( path => $config->indexing_directory );
+        my $munger = $config->search_content_munger_module;
+        if ( $munger ) {
+            eval {
+                eval "require $munger";
+                $plucene_args{content_munger} = sub {
+                    my $content = shift;
+                    return $munger->search_content_munger( $content );
+                };
+            };
+        }
+        $search = Wiki::Toolkit::Search::Plucene->new( %plucene_args );
     } else {
         require Wiki::Toolkit::Search::SII;
         require Search::InvertedIndex::DB::DB_File_SplitHash;
@@ -127,47 +141,39 @@ sub make_wiki_object {
                   if ( UNIVERSAL::isa( $_[0], "Wiki::Toolkit" ) ) {
                       shift; # just throw it away
                   }
+                  my $type = ( lc( $_[0] ) eq "category" ) ? "cat" : "loc";
                   my $link_title = $_[2] || "View all pages in $_[0] $_[1]";
-                  return qq(<a href="$script_name?action=index;index_type=) . uri_escape(lc($_[0])) . qq(;index_value=) . uri_escape($_[1]) . qq(">$link_title</a>);
+                  return qq(<a href="$script_name?action=index;$type=) . uri_escape( lc( $_[1] ) ) . qq(">$link_title</a>);
                 },
         qr/\@INDEX_LIST\s+\[\[(Category|Locale)\s+([^\]]+)]]/ =>
              sub {
                    my ($wiki, $type, $value) = @_;
-
-                   # We may be being called by Wiki::Toolkit::Plugin::Diff,
-                   # which doesn't know it has to pass us $wiki
-                   unless ( UNIVERSAL::isa( $wiki, "Wiki::Toolkit" ) ) {
-                       return "(unprocessed INDEX_LIST macro)";
-		   }
-
-                   my @nodes = sort $wiki->list_nodes_by_metadata(
-                       metadata_type  => $type,
-                       metadata_value => $value,
-                       ignore_case    => 1,
-                   );
-                   unless ( scalar @nodes ) {
-                       return "\n* No pages currently in "
-                              . lc($type) . " $value\n";
-                   }
-                   my $return = "\n";
-                   foreach my $node ( @nodes ) {
-                       $return .= "* "
-                               . $wiki->formatter->format_link(
-                                                                wiki => $wiki,
-                                                                link => $node,
-                                                              )
-                                . "\n";
-	           }
-                   return $return;
+                   return $class->do_index_list_macro(
+                       wiki => $wiki, type => $type, value => $value,
+                       include_prefix => 1 );
+                 },
+        qr/\@INDEX_LIST_NO_PREFIX\s+\[\[(Category|Locale)\s+([^\]]+)]]/ =>
+             sub {
+                   my ($wiki, $type, $value) = @_;
+                   return $class->do_index_list_macro(
+                       wiki => $wiki, type => $type, value => $value );
+                 },
+         qr/\@NODE_COUNT\s+\[\[(Category|Locale)\s+([^\]]+)]]/ =>
+             sub {
+                   my ($wiki, $type, $value) = @_;
+                   return $class->do_node_count(
+                       wiki => $wiki, type => $type, value => $value );
                  },
         qr/\@MAP_LINK\s+\[\[(Category|Locale)\s+([^\]|]+)\|?([^\]]+)?\]\]/ =>
                 sub {
                       if ( UNIVERSAL::isa( $_[0], "Wiki::Toolkit" ) ) {
                           shift; # don't need $wiki
                       }
+
+                      my $type = ( lc( $_[0] ) eq "category" ) ? "cat" : "loc";
                       my $link_title = $_[2]
                                        || "View map of pages in $_[0] $_[1]";
-                      return qq(<a href="$script_name?action=index;format=map;index_type=) . uri_escape(lc($_[0])) . qq(;index_value=) . uri_escape($_[1]) . qq(">$link_title</a>);
+                      return qq(<a href="$script_name?action=index;format=map;$type=) . uri_escape( lc( $_[1] ) ) . qq(">$link_title</a>);
                 },
         qr/\@RANDOM_PAGE_LINK(?:\s+\[\[(Category|Locale)\s+([^\]|]+)\|?([^\]]+)?\]\])?/ =>
                 sub {
@@ -186,61 +192,36 @@ sub make_wiki_object {
                       }
                       return qq(<a href="$link">$link_title</a>);
                 },
-        qr/\@INCLUDE_NODE\s+\[\[([^\]|]+)\]\]/ => 
+        qr/\@INCLUDE_NODE\s+\[\[([^\]|]+)\]\]/ =>
             sub {
                   my ($wiki, $node) = @_;
                   my %node_data = $wiki->retrieve_node( $node );
                   return $node_data{content};
                 },
-	qr/\@RSS\s+(.+)/ => sub {
-                    # We may be being called by Wiki::Toolkit::Plugin::Diff,
-                    # which doesn't know it has to pass us $wiki - and
-                    # we don't use it anyway.
-                    if ( UNIVERSAL::isa( $_[0], "Wiki::Toolkit" ) ) {
-                        shift; # just throw it away
-                    }
-
-                    my $url = shift;
-
-                    # The URL will already have been processed as an inline
-                    # link, so transform it back again.
-                    if ( $url =~ m/href="([^"]+)/ ) {
-                        $url = $1;
-                    }
-
-                    my $rss = Wiki::Toolkit::Plugin::RSS::Reader->new(url => $url);
-                    my @items = $rss->retrieve;
-
-                    # Ten items only at this till.
-                    $#items = 10 if $#items > 10;
-
-                    # Make a UseMod-formatted list with them - macros are
-                    # processed *before* UseMod formatting is applied but
-                    # *after* inline links like [http://foo/ bar]
-                    my $list = "\n";
-                    foreach (@items) {
-                        my $link        = $_->{link};
-                        my $title       = $_->{title};
-                        my $description = $_->{description};
-                        $list .= qq{* <a href="$link">$title</a>};
-                        $list .= " - $description" if $description;
-                        $list .= "\n";
-                    }
-                    $list .= "</ul>\n";
-        },
     );
 
+    my $custom_macro_module = $config->custom_macro_module;
+    if ( $custom_macro_module ) {
+        eval {
+            eval "require $custom_macro_module";
+            %macros = $custom_macro_module->custom_macros(macros => \%macros);
+        };
+    }
+
     my $formatter = Wiki::Toolkit::Formatter::UseMod->new(
         extended_links      => 1,
         implicit_links      => 0,
         allowed_tags        => [qw(a p b strong i em pre small img table td
                                    tr th br hr ul li center blockquote kbd
-                                   div code strike sub sup font)],
+                                   div code span strike sub sup font dl dt dd
+                                  )],
         macros              => \%macros,
         pass_wiki_to_macros => 1,
         node_prefix         => "$script_name?",
         edit_prefix         => "$script_name?action=edit;id=",
         munge_urls          => 1,
+        external_link_class => "external",
+        escape_url_commas   => 0,
     );
 
     my %conf = ( store     => $store,
@@ -251,6 +232,80 @@ sub make_wiki_object {
     return $wiki;
 }
 
+sub make_lucy_searcher {
+    my ( $class, %args ) = @_;
+    require Wiki::Toolkit::Search::Lucy;
+    my $config = $args{config};
+    my %lucy_args = (
+             path => $config->indexing_directory,
+             metadata_fields => [ qw( address category locale ) ],
+             boost => { title => 10 }, # empirically determined (test t/306)
+    );
+    my $munger = $config->search_content_munger_module;
+    if ( $munger ) {
+        eval {
+            eval "require $munger";
+            $lucy_args{content_munger} = sub {
+                my $content = shift;
+                return $munger->search_content_munger( $content );
+            };
+        };
+    }
+    return Wiki::Toolkit::Search::Lucy->new( %lucy_args );
+}
+
+sub do_index_list_macro {
+    my ( $class, %args ) = @_;
+    my ( $wiki, $type, $value, $include_prefix )
+        = @args{ qw( wiki type value include_prefix ) };
+
+    # We may be being called by Wiki::Toolkit::Plugin::Diff,
+    # which doesn't know it has to pass us $wiki
+    if ( !UNIVERSAL::isa( $wiki, "Wiki::Toolkit" ) ) {
+        if ( $args{include_prefix} ) {
+            return "(unprocessed INDEX_LIST macro)";
+        } else {
+            return "(unprocessed INDEX_LIST_NO_PREFIX macro)";
+        }
+    }
+
+    my @nodes = sort $wiki->list_nodes_by_metadata(
+        metadata_type  => $type,
+        metadata_value => $value,
+        ignore_case    => 1,
+    );
+    unless ( scalar @nodes ) {
+        return "\n* No pages currently in " . lc($type) . " $value\n";
+    }
+    my $return = "\n";
+    foreach my $node ( @nodes ) {
+        my $title = $node;
+        $title =~ s/^(Category|Locale) // unless $args{include_prefix};
+        $return .= "* "
+                . $wiki->formatter->format_link( wiki => $wiki,
+                                                 link => "$node|$title" )
+                . "\n";
+    }
+    return $return;
+}
+sub do_node_count {
+    my ( $class, %args ) = @_;
+    my ( $wiki, $type, $value )
+        = @args{ qw( wiki type value ) };
+
+    # We may be being called by Wiki::Toolkit::Plugin::Diff,
+    # which doesn't know it has to pass us $wiki
+    if ( !UNIVERSAL::isa( $wiki, "Wiki::Toolkit" ) ) {
+            return "(unprocessed NODE_COUNT macro)";
+        }
+
+    my $num_nodes = scalar $wiki->list_nodes_by_metadata(
+        metadata_type  => $type,
+        metadata_value => $value,
+        ignore_case    => 1,
+    );
+    return $num_nodes;
+}
 =item B<get_wgs84_coords>
 
 Returns coordinate data suitable for use with Google Maps (and other GIS
@@ -299,7 +354,7 @@ sub get_wgs84_coords {
                     return undef;
                 }
 
-                my ($lat,$long,$h) = 
+                my ($lat,$long,$h) =
                     Geo::HelmertTransform::convert_datum($datum_helper,$wgs84_helper,$oldlat,$oldlong,0);
                 return ($long,$lat);
             };
@@ -307,9 +362,9 @@ sub get_wgs84_coords {
     }
     # Give up, return undef
     unless($helmert) {
-       return undef; 
+       return undef;
     }
-    
+
 
     if ($config->geo_handler == 1) {
         # Do conversion here
@@ -329,6 +384,94 @@ sub get_wgs84_coords {
     }
 }
 
+=item B<get_wgs84_min_max>
+
+Given a set of WGS84 coordinate data, returns the minimum, maximum,
+and centre latitude and longitude.
+
+    %data = OpenGuides::Utils->get_wgs84_min_max(
+        nodes => [
+                   { wgs84_lat => 51.1, wgs84_long => 1.1 },
+                   { wgs84_lat => 51.2, wgs84_long => 1.2 },
+                 ]
+    );
+    print "Top right-hand corner is $data{max_lat}, $data{max_long}";
+    print "Centre point is $data{centre_lat}, $data{centre_long}";
+
+The hashes in the C<nodes> argument can include other key/value pairs;
+these will just be ignored.
+
+Returns false if it can't find any valid geodata in the nodes.
+
+=cut
+
+sub get_wgs84_min_max {
+    my ( $self, %args ) = @_;
+    my @nodes = @{$args{nodes}};
+
+    my @lats  = sort
+                grep { defined $_ && /^[-.\d]+$/ }
+                map { $_->{wgs84_lat} }
+                @nodes;
+    my @longs = sort
+                grep { defined $_ && /^[-.\d]+$/ }
+                map { $_->{wgs84_long} }
+                @nodes;
+
+    if ( !scalar @lats || !scalar @longs ) {
+        return;
+    }
+
+    my %data = ( min_lat  => $lats[0],  max_lat  => $lats[$#lats],
+                 min_long => $longs[0], max_long => $longs[$#longs] );
+    $data{centre_lat} = ( $data{min_lat} + $data{max_lat} ) / 2;
+    $data{centre_long} = ( $data{min_long} + $data{max_long} ) / 2;
+    return %data;
+}
+
+=item B<get_index_page_description>
+
+    $tt_vars{page_description} =
+        OpenGuides::Utils->get_index_page_description(
+            format => "map",
+            criteria => [ type => "locale", value => "croydon" ],
+    );
+
+Returns a sentence that can be used as a summary of what's shown on an
+index page.
+
+=cut
+
+sub get_index_page_description {
+    my ( $class, %args ) = @_;
+    my $desc = ( $args{format} eq "map" ) ? "Map" : "List";
+    $desc .= " of all our pages";
+
+    my ( @cats, @locs );
+    foreach my $criterion ( @{$args{criteria}} ) {
+        my ( $type, $name ) = ( $criterion->{type}, $criterion->{name} );
+        if ( $type eq "category" ) {
+            $name =~ s/Category //;
+            push @cats, $name;
+        } else {
+            $name =~ s/Locale //;
+            push @locs, $name;
+        }
+    }
+
+    if ( scalar @cats ) {
+        $desc .= " labelled with: " . join( ", ", @cats );
+        if ( scalar @locs ) {
+            $desc .= ", and";
+        }
+    }
+    if ( scalar @locs ) {
+        $desc .= " located in: " . join( ", ", @locs );
+    }
+    $desc .= ".";
+    return $desc;
+}
+
 =item B<detect_redirect>
 
     $redir = OpenGuides::Utils->detect_redirect( content => "foo" );
@@ -376,7 +519,6 @@ sub validate_edit {
     return \@fails if @fails;
 
     # Now do our real validation
-    # Numeric tests
     foreach my $var (qw(os_x os_y)) {
         if ($q->param($var) and $q->param($var) !~ /^-?\d+$/) {
             push @fails, "$var must be integer, was: " . $q->param($var);
@@ -389,10 +531,57 @@ sub validate_edit {
         }
     }
 
+    if ( $q->param('website') and $q->param('website') ne 'http://' ) {
+        unless ( is_web_uri( scalar $q->param('website') ) ) {
+            push @fails, $q->param('website') . ' is not a valid web URI';
+        }
+    }
+
     return \@fails;
 
 };
 
+=item B<parse_change_comment>
+
+    my $change_comment = parse_change_comment($string, $base_url);
+
+Given a base URL (for example, C<http://example.com/wiki.cgi?>), takes a string,
+replaces C<[[page]]> and C<[[page|titled link]]> with
+
+    <a href="http://example.com/wiki.cgi?page">page</a>
+
+and
+
+    <a href="http://example.com/wiki.cgi?page">titled link</a>
+
+respectively, and returns it. This is a limited subset of wiki markup suitable for
+use in page change comments.
+
+=cut
+
+sub parse_change_comment {
+    my ($comment, $base_url) = @_;
+
+    my @links = $comment =~ m{\[\[(.*?)\]\]}g;
+
+    # It's not all that great having to reinvent the wheel in this way, but
+    # Text::WikiFormat won't let you specify the subset of wiki notation that
+    # you're interested in. C'est la vie.
+    foreach (@links) {
+        if (/(.*?)\|(.*)/) {
+            my ($page, $title) = ($1, $2);
+            $comment =~ s{\[\[$page\|$title\]\]}
+                         {<a href="$base_url$page">$title</a>};
+        } else {
+            my $page = $_;
+            $comment =~ s{\[\[$page\]\]}
+                         {<a href="$base_url$page">$page</a>};
+        }
+    }
+
+    return $comment;
+}
+
 =item B<send_email>
 
     eval { OpenGuides::Utils->send_email(
@@ -421,6 +610,7 @@ a string instead of being sent by email.
 
 =cut
 
+
 sub send_email {
     my ( $self, %args ) = @_;
     my $config = $args{config} or die "config argument not supplied";
@@ -428,7 +618,7 @@ sub send_email {
     @to = @{$args{to}} if $args{to};
     my @bcc;
     push @to, $config->contact_email if $args{admin};
-    die "No recipients specified" unless scalar @to;
+    die "No recipients specified" unless $to[0];
     die "No subject specified" unless $args{subject};
     die "No body specified" unless $args{body};
     my $to_str = join ',', @to;
@@ -449,6 +639,35 @@ sub send_email {
     }
 }
 
+=item B<in_moderate_whitelist>
+
+ if (OpenGuides::Utils->in_moderate_whitelist( '127.0.0.1' )) {
+     # skip moderation and apply new verson to published site
+ }
+
+Admins can supply a comma separated list of IP addresses or CIDR-notation
+subnets indicating the hosts which can bypass enforced moderation. Any
+values which cannot be parsed by C<NetAddr::IP> will be ignored.
+
+=cut
+
+sub in_moderate_whitelist {
+    my ($self, $config, $ip) = @_;
+    return undef if not defined $ip;
+
+    # create NetAddr::IP object of the test IP
+    my $addr = Net::Netmask->new2($ip) or return undef;
+
+    # load the configured whitelist
+    my @whitelist
+        = split ',', $config->moderate_whitelist;
+
+    # test each entry in the whitelist
+    return eval{
+        first { Net::Netmask->new2($_)->match($addr->base) } @whitelist
+    };
+}
+
 =back
 
 =head1 AUTHOR
@@ -457,7 +676,7 @@ The OpenGuides Project (openguides-dev@lists.openguides.org)
 
 =head1 COPYRIGHT
 
-     Copyright (C) 2003-2007 The OpenGuides Project.  All Rights Reserved.
+     Copyright (C) 2003-2013 The OpenGuides Project.  All Rights Reserved.
 
 This module is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
